/*Copyright 2009 Jose C. Bandeira R. Cardoso

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package EsforcosModelo;

import EsforcosControle.*;        
import javax.swing.*;



public class PerfilEsforcos {

    /*Essa classe calcula a resistncia  deformao, em funo da temperatura, 
     *composio qumica e velocidade de deformao. Depois calcula o fator 
     *geomtrico, isto  o quanto a fora de separao entre os cilindros 
     *aumentada em funo do estado triaxial de tenses no passe. 
     *Esse esforo multiplicado pela rea de contato resulta na fora de
     *laminao. Para as vigas, essa fora  calculada separadamente para as abas e para a alma,
     *resultando depois numa fora total de separao horizontal e outra vertical. 
     *O torque para as abas  o produto do brao de alavanca (o raio do cilindro) 
     *pela fora de atrito do material com os cilindros. Essa ltima  estimada como
     *a tenso cizalhante na temperatura de laminao vezes a rea de contato da aba
     *com os cilindros. A potncia  o torque vezes a velocidade angular dos cilindros.
     *Para as almas, o clculo do torque  o usual da fora vezes o arco de contato.
     *Os torques e potncias totais so a soma das componentes para as abas e alma.
     *
     *This class calculates the resistance to deformation, as a function of the
     *temperature, chemical composition and deformation speed. After that it calculates
     *the geometrical factor, that is, how much the separation force between rolls 
     *is increased by the triaxial state of stresses in the pass. That load, multiplied
     *by the area of contact, results in the rolling force. For the beams and channels, 
     *this force is calculated separately for the web and legs, resulting in a
     *horizontal and vertical separating force.
     *Torque is, for the legs, the product of the lever arm (roll radius) by the friction force.
     *That later is estimated as the shear stress limit at the temperature multiplied
     *by the area of contact arc. Power is torque x roll angular speed. 
     *For the webs, torque is calculated is the usual: force x lenght of contact arc.
     *Total torques and power are the sum of the componets for the web and legs.   
     *
     */
    
    private static ModeloGeometrico modelo;
    private static Equacao2Grau equacao;
    private static final int EKELUND = 2, MAUK = 1, TSELIKOV = 0, MISAKA = 3;
    private static int metodoDeForca;
    private static double kF, fator, deformacaoUniformeAba, velocidadeDeformacaoAba, 
            err, deformacaoUniformeAlma, velocidadeDeformacaoAlma, kFAba, kFAlma;
    
    /** Creates a new instance of Esforcos */
    public PerfilEsforcos() {        
        
        modelo = SegundoTratador.getModelo();
        metodoDeForca = modelo.getMetodo();       
        
        deformacaoUniformeAba = Math.log(modelo.getEspessuraInicialAba() / modelo.getEspessuraFinalAba());
        velocidadeDeformacaoAba = modelo.getVelocidade() * deformacaoUniformeAba / ( modelo.arcoDeContatoAba() / 1000 );
        
        deformacaoUniformeAlma = Math.log(modelo.getEspessuraInicialAlma() / modelo.getEspessuraFinalAlma());
        velocidadeDeformacaoAlma = modelo.getVelocidade() * deformacaoUniformeAlma / ( modelo.arcoDeContatoAlma() / 1000 );
        
        if ( metodoDeForca == EKELUND || metodoDeForca == TSELIKOV )
            resistenciaDeformacaoEkelund();
        
        else if ( metodoDeForca == MAUK )
            resistenciaDeformacaoMauk();
        
        else if ( metodoDeForca == MISAKA )
            resistenciaDeformacaoMisaka();
        
        else kF=7;
        
        
    }
    
    public static void resistenciaDeformacaoMauk(){             
        
        double A = 4.59462904;
        double B = .179599573;
        double C = .0196427786;
        double D = .0566336862000012;
        double E = .00235420612;
        
        kFAba = Math.exp( A - B * deformacaoUniformeAba + C * Math.log( deformacaoUniformeAba ) + 
                D * Math.log ( velocidadeDeformacaoAba ) - E * modelo.getTemperatura());
        
        kFAlma = Math.exp( A - B * deformacaoUniformeAlma + C * Math.log( deformacaoUniformeAlma ) + 
                D * Math.log ( velocidadeDeformacaoAlma ) - E * modelo.getTemperatura());
        
    }
    
    public static void resistenciaDeformacaoEkelund(){
        
        double epsilon = 0.01 * ( 20164 - 7.89 * ( 9.0 / 5.0 * modelo.getTemperatura() + 32 ) ) / 2240;
        double epsilonMetrico = epsilon * 1.01605 / ( 25.4001 * 25.4001 );
       
        double sigma = 100 * epsilonMetrico * ( 1.4 + modelo.getCarbono() + modelo.getManganes() + 0.3 * modelo.getCromo());
        kFAba = ( sigma + 2 * modelo.getVelocidade() * epsilonMetrico * Math.sqrt(modelo.deltaHAba() / ( modelo.getDiametroCorrigidoAba() / 2 ) )/
        modelo.somaHAba() ) * 1000;
       
        kFAlma = ( sigma + 2 * modelo.getVelocidade() * epsilonMetrico * Math.sqrt(modelo.deltaHAlma() / ( modelo.getDiametroHorizontal() / 2 ) )/
        modelo.somaHAlma() ) * 1000;
    }
    
    public static void resistenciaDeformacaoMisaka(){
        
        double C = modelo.getCarbono();
        double beta = 0.126 -  1.75 * C  +  0.594 * Math.pow ( C , 2 )  + 
                ( 2831 +  2968 * C  -  1120 * Math.pow ( C , 2 ) ) / ( modelo.getTemperatura() + 273 );
        
        kFAba = Math.pow ( deformacaoUniformeAba ,  0.21  ) * Math.pow ( velocidadeDeformacaoAba , 0.13 ) * Math.exp(beta); 
        kFAba = Math.pow ( deformacaoUniformeAlma ,  0.21  ) * Math.pow ( velocidadeDeformacaoAlma , 0.13 ) * Math.exp(beta); 
        
    }
    
    public static double fatorGeometricoAba(){
        
        if ( metodoDeForca == MAUK || metodoDeForca == EKELUND )
            return 1 + modelo.getFatorDeFormaAba();
        
        else if ( metodoDeForca == TSELIKOV ){
            tselikovAba();
            return fator;      
        }
        
        else {
            nakajimaAba();
            return fator; 
            
        }
    }
    
    public static double fatorGeometricoAlma(){
        
        if ( metodoDeForca == MAUK || metodoDeForca == EKELUND )
            return 1 + modelo.getFatorDeFormaAlma();
        
        else if ( metodoDeForca == TSELIKOV ){
            tselikovAlma();
            return fator;      
        }
        
        else {
            nakajimaAlma();
            return fator; 
            
        }
    }       
     
    public static double forcaVertical(){
        
        
        return 2 * forcaAba() * Math.sin(modelo.getAnguloInterno()) + forcaAlma();
    }
    
    public static double forcaHorizontal(){
        
        return 2 * forcaAba() * Math.cos(modelo.getAnguloInterno());
        
    }
    
    
    public static double forcaAba(){        
        
        
        //return fatorGeometrico() * modelo.getLargura()  
                 //* modelo.arcoDeContato() * kF * Math.sin(modelo.getAngulo());        
        return fatorGeometricoAba() * modelo.getLarguraAba()  
                 * modelo.arcoDeContatoAba() * kFAba;      
    } 
    
    public static double forcaAlma(){        
        
        if(modelo.getLarguraAlma() != 0)      
            return fatorGeometricoAlma() * modelo.getLarguraAlma()  
                 * modelo.arcoDeContatoAlma() * kFAlma;  
        else 
            return 0;
    } 
    
    
    public static double torque(){
        
        return 2 * torqueAba() + torqueAlma();
        
    }
    
    public static double torqueAba(){        
        
        if(TratadorDados.getPerfil() == TratadorDados.VIGAIUNI || TratadorDados.getPerfil() == TratadorDados.VIGAUUNI)
            return forcaAba() * modelo.arcoDeContatoAba() / 1000; 
        else
            return kFAba/2 * modelo.getLarguraAba() * modelo.arcoDeContatoAba() * modelo.getDiametroHorizontal()/2/1000; 
        //return 2 * forca() * modelo.coeficienteDeAtrito() * modelo.getDiametro()/2/1000; 
    }
    
    public static double torqueAlma(){        
        
        if(forcaAlma() != 0)
            return forcaAlma() * modelo.arcoDeContatoAlma() / 1000; 
        else
            return 0;
       
    }
    
    public static double potencia(){
        
        return 2 * potenciaAba() + potenciaAlma();        
    }
    
    
    public static double potenciaAba(){
        
        return torqueAba() * 2.0 * Math.PI / 60 * modelo.getRotacao() * 9.8 / 1000;     
                
    }
    
    public static double potenciaAlma(){
        
        if(torqueAlma() != 0)
            return torqueAlma() * 2.0 * Math.PI / 60 * modelo.getRotacao() * 9.8 / 1000;    
        else
            return 0;
                
    }
    
    public static void tselikovAba(){
        
        try{
            
        
        Colina colina ;
        colina = new Colina(modelo.getDiametroCorrigidoAba(), modelo.getEspessuraInicialAba(), modelo.getEspessuraFinalAba(), modelo.coeficienteDeAtrito(), 0);
        
        
        fator = colina.fatorGeometrico();
        
        }
        
        catch ( Exception exception ){
            
            JOptionPane.showMessageDialog(null," Erro em tselikov() ","Erro em Esforcos.java",JOptionPane.ERROR_MESSAGE);
            
        }
        
    }
    
    public static void tselikovAlma(){
        
        try{
            
        
        Colina colina ;
        colina = new Colina(modelo.getDiametroHorizontal(), modelo.getEspessuraInicialAlma(), modelo.getEspessuraFinalAlma(), modelo.coeficienteDeAtrito(), 0);
        
        
        fator = colina.fatorGeometrico();
        
        }
        
        catch ( Exception exception ){
            
            JOptionPane.showMessageDialog(null," Erro em tselikov() ","Erro em Esforcos.java",JOptionPane.ERROR_MESSAGE);
            
        }
        
    }
    
    public static void nakajimaAba(){
        
        double alturaMedia = ( modelo.getEspessuraInicialAba() + 2 * modelo.getEspessuraFinalAba() ) / 3;
        fator = 0.25 * modelo.arcoDeContatoAba() / alturaMedia + 0.21 * alturaMedia / modelo.arcoDeContatoAba() + 0.6;
    }
    
    public static void nakajimaAlma(){
        
        double alturaMedia = ( modelo.getEspessuraInicialAlma() + 2 * modelo.getEspessuraFinalAlma() ) / 3;
        fator = 0.25 * modelo.arcoDeContatoAlma() / alturaMedia + 0.21 * alturaMedia / modelo.arcoDeContatoAlma() + 0.6;
    }
}